@@ -49,20 +49,12 @@
 # TODO: Change the parameter 'rev_id' to 'revision_id' to be consistent with
 # the rest of the code; add a deprecation of the old name.
 
-import os
-import re
-import sys
-import time
-
-from cStringIO import StringIO
-
 from bzrlib import (
     debug,
     errors,
     revision,
     trace,
     tree,
-    xml_serializer,
     )
 from bzrlib.branch import Branch
 from bzrlib.cleanup import OperationWithCleanups
@@ -72,20 +64,13 @@ from bzrlib.errors import (BzrError, PointlessCommit,
                            StrictCommitFailed
                            )
 from bzrlib.osutils import (get_user_encoding,
-                            kind_marker, isdir,isfile, is_inside_any,
-                            is_inside_or_parent_of_any,
+                            is_inside_any,
                             minimum_path_selection,
-                            quotefn, sha_file, split_lines,
                             splitpath,
                             )
-from bzrlib.testament import Testament
-from bzrlib.trace import mutter, note, warning, is_quiet
+from bzrlib.trace import mutter, note, is_quiet
 from bzrlib.inventory import Inventory, InventoryEntry, make_entry
 from bzrlib import symbol_versioning
-from bzrlib.symbol_versioning import (deprecated_passed,
-        deprecated_function,
-        DEPRECATED_PARAMETER)
-from bzrlib.workingtree import WorkingTree
 from bzrlib.urlutils import unescape_for_display
 import bzrlib.ui
 
@@ -147,6 +132,12 @@ class ReportCommitToLog(NullCommitReporter):
 
     def completed(self, revno, rev_id):
         self._note('Committed revision %d.', revno)
+        # self._note goes to the console too; so while we want to log the
+        # rev_id, we can't trivially only log it. (See bug 526425). Long
+        # term we should rearrange the reporting structure, but for now
+        # we just mutter seperately. We mutter the revid and revno together
+        # so that concurrent bzr invocations won't lead to confusion.
+        mutter('Committed revid %s as revno %d.', rev_id, revno)
 
     def deleted(self, path):
         self._note('deleted %s', path)
@@ -183,6 +174,43 @@ class Commit(object):
         self.reporter = reporter
         self.config = config
 
+    @staticmethod
+    def update_revprops(revprops, branch, authors=None, author=None,
+                        local=False, possible_master_transports=None):
+        if revprops is None:
+            revprops = {}
+        if possible_master_transports is None:
+            possible_master_transports = []
+        if not 'branch-nick' in revprops:
+            revprops['branch-nick'] = branch._get_nick(
+                local,
+                possible_master_transports)
+        if authors is not None:
+            if author is not None:
+                raise AssertionError('Specifying both author and authors '
+                        'is not allowed. Specify just authors instead')
+            if 'author' in revprops or 'authors' in revprops:
+                # XXX: maybe we should just accept one of them?
+                raise AssertionError('author property given twice')
+            if authors:
+                for individual in authors:
+                    if '\n' in individual:
+                        raise AssertionError('\\n is not a valid character '
+                                'in an author identity')
+                revprops['authors'] = '\n'.join(authors)
+        if author is not None:
+            symbol_versioning.warn('The parameter author was deprecated'
+                   ' in version 1.13. Use authors instead',
+                   DeprecationWarning)
+            if 'author' in revprops or 'authors' in revprops:
+                # XXX: maybe we should just accept one of them?
+                raise AssertionError('author property given twice')
+            if '\n' in author:
+                raise AssertionError('\\n is not a valid character '
+                        'in an author identity')
+            revprops['authors'] = author
+        return revprops
+
     def commit(self,
                message=None,
                timestamp=None,
@@ -236,6 +264,9 @@ class Commit(object):
             commit.
         """
         operation = OperationWithCleanups(self._commit)
+        self.revprops = revprops or {}
+        # XXX: Can be set on __init__ or passed in - this is a bit ugly.
+        self.config = config or self.config
         return operation.run(
                message=message,
                timestamp=timestamp,
@@ -246,19 +277,17 @@ class Commit(object):
                allow_pointless=allow_pointless,
                strict=strict,
                verbose=verbose,
-               revprops=revprops,
                working_tree=working_tree,
                local=local,
                reporter=reporter,
-               config=config,
                message_callback=message_callback,
                recursive=recursive,
                exclude=exclude,
                possible_master_transports=possible_master_transports)
 
     def _commit(self, operation, message, timestamp, timezone, committer,
-            specific_files, rev_id, allow_pointless, strict, verbose, revprops,
-            working_tree, local, reporter, config, message_callback, recursive,
+            specific_files, rev_id, allow_pointless, strict, verbose,
+            working_tree, local, reporter, message_callback, recursive,
             exclude, possible_master_transports):
         mutter('preparing to commit')
 
@@ -299,7 +328,6 @@ class Commit(object):
             self.specific_files = None
             
         self.allow_pointless = allow_pointless
-        self.revprops = revprops
         self.message_callback = message_callback
         self.timestamp = timestamp
         self.timezone = timezone
@@ -371,7 +399,7 @@ class Commit(object):
         # Collect the changes
         self._set_progress_stage("Collecting changes", counter=True)
         self.builder = self.branch.get_commit_builder(self.parents,
-            self.config, timestamp, timezone, committer, revprops, rev_id)
+            self.config, timestamp, timezone, committer, self.revprops, rev_id)
 
         try:
             self.builder.will_record_deletes()
