@@ -1685,12 +1685,18 @@ class GraphThunkIdsToKeys(object):
     def __init__(self, graph):
         self._graph = graph
 
+    def topo_sort(self):
+        return [r for (r,) in self._graph.topo_sort()]
+
     def heads(self, ids):
         """See Graph.heads()"""
         as_keys = [(i,) for i in ids]
         head_keys = self._graph.heads(as_keys)
         return set([h[0] for h in head_keys])
 
+    def merge_sort(self, tip_revision):
+        return self._graph.merge_sort((tip_revision,))
+
 
 _counters = [0,0,0,0,0,0,0]
 try:
