@@ -120,7 +120,6 @@ from bzrlib.errors import (
         LockBreakMismatch,
         LockBroken,
         LockContention,
-        LockCorrupt,
         LockFailed,
         LockNotHeld,
         NoSuchFile,
@@ -152,7 +151,7 @@ from bzrlib import rio
 # files/dirs created.
 
 
-_DEFAULT_TIMEOUT_SECONDS = 300
+_DEFAULT_TIMEOUT_SECONDS = 30
 _DEFAULT_POLL_SECONDS = 1.0
 
 
@@ -349,13 +348,7 @@ class LockDir(lock.Lock):
         it possibly being still active.
         """
         self._check_not_locked()
-        try:
-            holder_info = self.peek()
-        except LockCorrupt, e:
-            # The lock info is corrupt.
-            if bzrlib.ui.ui_factory.get_boolean("Break (corrupt %r)" % (self,)):
-                self.force_break_corrupt(e.file_data)
-            return
+        holder_info = self.peek()
         if holder_info is not None:
             lock_info = '\n'.join(self._format_lock_info(holder_info))
             if bzrlib.ui.ui_factory.get_boolean("Break %s" % lock_info):
@@ -402,35 +395,6 @@ class LockDir(lock.Lock):
         for hook in self.hooks['lock_broken']:
             hook(result)
 
-    def force_break_corrupt(self, corrupt_info_lines):
-        """Release a lock that has been corrupted.
-        
-        This is very similar to force_break, it except it doesn't assume that
-        self.peek() can work.
-        
-        :param corrupt_info_lines: the lines of the corrupted info file, used
-            to check that the lock hasn't changed between reading the (corrupt)
-            info file and calling force_break_corrupt.
-        """
-        # XXX: this copes with unparseable info files, but what about missing
-        # info files?  Or missing lock dirs?
-        self._check_not_locked()
-        tmpname = '%s/broken.%s.tmp' % (self.path, rand_chars(20))
-        self.transport.rename(self._held_dir, tmpname)
-        # check that we actually broke the right lock, not someone else;
-        # there's a small race window between checking it and doing the
-        # rename.
-        broken_info_path = tmpname + self.__INFO_NAME
-        f = self.transport.get(broken_info_path)
-        broken_lines = f.readlines()
-        if broken_lines != corrupt_info_lines:
-            raise LockBreakMismatch(self, broken_lines, corrupt_info_lines)
-        self.transport.delete(broken_info_path)
-        self.transport.rmdir(tmpname)
-        result = lock.LockResult(self.transport.abspath(self.path))
-        for hook in self.hooks['lock_broken']:
-            hook(result)
-
     def _check_not_locked(self):
         """If the lock is held by this instance, raise an error."""
         if self._lock_held:
@@ -483,9 +447,9 @@ class LockDir(lock.Lock):
         # XXX: is creating this here inefficient?
         config = bzrlib.config.GlobalConfig()
         try:
-            user = config.user_email()
-        except errors.NoEmailInUsername:
             user = config.username()
+        except errors.NoWhoami:
+            user = osutils.getuser_unicode()
         s = rio.Stanza(hostname=get_host_name(),
                    pid=str(os.getpid()),
                    start_time=str(int(time.time())),
@@ -495,13 +459,7 @@ class LockDir(lock.Lock):
         return s.to_string()
 
     def _parse_info(self, info_bytes):
-        lines = osutils.split_lines(info_bytes)
-        try:
-            stanza = rio.read_stanza(lines)
-        except ValueError, e:
-            mutter('Corrupt lock info file: %r', lines)
-            raise LockCorrupt("could not parse lock info file: " + str(e),
-                              lines)
+        stanza = rio.read_stanza(osutils.split_lines(info_bytes))
         if stanza is None:
             # see bug 185013; we fairly often end up with the info file being
             # empty after an interruption; we could log a message here but
@@ -581,24 +539,27 @@ class LockDir(lock.Lock):
                 if deadline_str is None:
                     deadline_str = time.strftime('%H:%M:%S',
                                                  time.localtime(deadline))
+                # As local lock urls are correct we display them.
+                # We avoid displaying remote lock urls.
                 lock_url = self.transport.abspath(self.path)
-                # See <https://bugs.launchpad.net/bzr/+bug/250451>
-                # the URL here is sometimes not one that is useful to the
-                # user, perhaps being wrapped in a lp-%d or chroot decorator,
-                # especially if this error is issued from the server.
-                self._report_function('%s %s\n'
-                    '%s\n' # held by
-                    '%s\n' # locked ... ago
-                    'Will continue to try until %s, unless '
-                    'you press Ctrl-C.\n'
-                    'See "bzr help break-lock" for more.',
-                    start,
-                    formatted_info[0],
-                    formatted_info[1],
-                    formatted_info[2],
-                    deadline_str,
-                    )
-
+                if lock_url.startswith('file://'):
+                    lock_url = lock_url.split('.bzr/')[0]
+                else:
+                    lock_url = ''
+                user, hostname, pid, time_ago = formatted_info
+                msg = ('%s lock %s '        # lock_url
+                    'held by '              # start
+                    '%s\n'                  # user
+                    'at %s '                # hostname
+                    '[process #%s], '       # pid
+                    'acquired %s.')         # time ago
+                msg_args = [start, lock_url, user, hostname, pid, time_ago]
+                if timeout > 0:
+                    msg += ('\nWill continue to try until %s, unless '
+                        'you press Ctrl-C.')
+                    msg_args.append(deadline_str)
+                msg += '\nSee "bzr help break-lock" for more.'
+                self._report_function(msg, *msg_args)
             if (max_attempts is not None) and (attempt_count >= max_attempts):
                 self._trace("exceeded %d attempts")
                 raise LockContention(self)
@@ -606,8 +567,11 @@ class LockDir(lock.Lock):
                 self._trace("waiting %ss", poll)
                 time.sleep(poll)
             else:
+                # As timeout is always 0 for remote locks
+                # this block is applicable only for local
+                # lock contention
                 self._trace("timeout after waiting %ss", timeout)
-                raise LockContention(self)
+                raise LockContention('(local)', lock_url)
 
     def leave_in_place(self):
         self._locked_via_token = True
@@ -658,17 +622,19 @@ class LockDir(lock.Lock):
 
     def _format_lock_info(self, info):
         """Turn the contents of peek() into something for the user"""
-        lock_url = self.transport.abspath(self.path)
         start_time = info.get('start_time')
         if start_time is None:
             time_ago = '(unknown)'
         else:
             time_ago = format_delta(time.time() - int(info['start_time']))
+        user = info.get('user', '<unknown>')
+        hostname = info.get('hostname', '<unknown>')
+        pid = info.get('pid', '<unknown>')
         return [
-            'lock %s' % (lock_url,),
-            'held by %s on host %s [process #%s]' %
-                tuple([info.get(x, '<unknown>') for x in ['user', 'hostname', 'pid']]),
-            'locked %s' % (time_ago,),
+            user,
+            hostname,
+            pid,
+            time_ago,
             ]
 
     def validate_token(self, token):
