@@ -813,8 +813,14 @@ class RevisionSpec_branch(RevisionSpec):
         revision_b = other_branch.last_revision()
         if revision_b in (None, revision.NULL_REVISION):
             raise errors.NoCommits(other_branch)
-        # pull in the remote revisions so we can diff
-        branch.fetch(other_branch, revision_b)
+        if branch is None:
+            branch = other_branch
+        else:
+            try:
+                # pull in the remote revisions so we can diff
+                branch.fetch(other_branch, revision_b)
+            except errors.ReadOnlyError:
+                branch = other_branch
         try:
             revno = branch.revision_id_to_revno(revision_b)
         except errors.NoSuchRevision:
@@ -840,6 +846,12 @@ class RevisionSpec_branch(RevisionSpec):
             raise errors.NoCommits(other_branch)
         return other_branch.repository.revision_tree(last_revision)
 
+    def needs_branch(self):
+        return False
+
+    def get_branch(self):
+        return self.spec
+
 
 
 class RevisionSpec_submit(RevisionSpec_ancestor):
