@@ -24,15 +24,14 @@ over SSH), and pass them to and from the protocol logic.  See the overview in
 bzrlib/transport/smart/__init__.py.
 """
 
-import errno
 import os
-import socket
 import sys
 import urllib
 
+import bzrlib
 from bzrlib.lazy_import import lazy_import
 lazy_import(globals(), """
-import atexit
+import socket
 import thread
 import weakref
 
@@ -47,14 +46,13 @@ from bzrlib import (
 from bzrlib.smart import client, protocol, request, vfs
 from bzrlib.transport import ssh
 """)
-#usually already imported, and getting IllegalScoperReplacer on it here.
 from bzrlib import osutils
 
-# We must not read any more than 64k at a time so we don't risk "no buffer
-# space available" errors on some platforms.  Windows in particular is likely
-# to give error 10053 or 10055 if we read more than 64k from a socket.
-_MAX_READ_SIZE = 64 * 1024
-
+# Throughout this module buffer size parameters are either limited to be at
+# most _MAX_READ_SIZE, or are ignored and _MAX_READ_SIZE is used instead.
+# For this module's purposes, MAX_SOCKET_CHUNK is a reasonable size for reads
+# from non-sockets as well.
+_MAX_READ_SIZE = osutils.MAX_SOCKET_CHUNK
 
 def _get_protocol_factory_for_bytes(bytes):
     """Determine the right protocol factory for 'bytes'.
@@ -276,9 +274,9 @@ class SmartServerSocketStreamMedium(SmartServerStreamMedium):
     def _serve_one_request_unguarded(self, protocol):
         while protocol.next_read_size():
             # We can safely try to read large chunks.  If there is less data
-            # than _MAX_READ_SIZE ready, the socket wil just return a short
-            # read immediately rather than block.
-            bytes = self.read_bytes(_MAX_READ_SIZE)
+            # than MAX_SOCKET_CHUNK ready, the socket will just return a
+            # short read immediately rather than block.
+            bytes = self.read_bytes(osutils.MAX_SOCKET_CHUNK)
             if bytes == '':
                 self.finished = True
                 return
@@ -287,13 +285,13 @@ class SmartServerSocketStreamMedium(SmartServerStreamMedium):
         self._push_back(protocol.unused_data)
 
     def _read_bytes(self, desired_count):
-        return _read_bytes_from_socket(
-            self.socket.recv, desired_count, self._report_activity)
+        return osutils.read_bytes_from_socket(
+            self.socket, self._report_activity)
 
     def terminate_due_to_error(self):
         # TODO: This should log to a server log file, but no such thing
         # exists yet.  Andrew Bennetts 2006-09-29.
-        osutils.until_no_eintr(self.socket.close)
+        self.socket.close()
         self.finished = True
 
     def _write_out(self, bytes):
@@ -334,27 +332,27 @@ class SmartServerPipeStreamMedium(SmartServerStreamMedium):
             bytes_to_read = protocol.next_read_size()
             if bytes_to_read == 0:
                 # Finished serving this request.
-                osutils.until_no_eintr(self._out.flush)
+                self._out.flush()
                 return
             bytes = self.read_bytes(bytes_to_read)
             if bytes == '':
                 # Connection has been closed.
                 self.finished = True
-                osutils.until_no_eintr(self._out.flush)
+                self._out.flush()
                 return
             protocol.accept_bytes(bytes)
 
     def _read_bytes(self, desired_count):
-        return osutils.until_no_eintr(self._in.read, desired_count)
+        return self._in.read(desired_count)
 
     def terminate_due_to_error(self):
         # TODO: This should log to a server log file, but no such thing
         # exists yet.  Andrew Bennetts 2006-09-29.
-        osutils.until_no_eintr(self._out.close)
+        self._out.close()
         self.finished = True
 
     def _write_out(self, bytes):
-        osutils.until_no_eintr(self._out.write, bytes)
+        self._out.write(bytes)
 
 
 class SmartClientMediumRequest(object):
@@ -496,16 +494,16 @@ class SmartClientMediumRequest(object):
 class _DebugCounter(object):
     """An object that counts the HPSS calls made to each client medium.
 
-    When a medium is garbage-collected, or failing that when atexit functions
-    are run, the total number of calls made on that medium are reported via
-    trace.note.
+    When a medium is garbage-collected, or failing that when
+    bzrlib.global_state exits, the total number of calls made on that medium
+    are reported via trace.note.
     """
 
     def __init__(self):
         self.counts = weakref.WeakKeyDictionary()
         client._SmartClient.hooks.install_named_hook(
             'call', self.increment_call_count, 'hpss call counter')
-        atexit.register(self.flush_all)
+        bzrlib.global_state.cleanups.add_cleanup(self.flush_all)
 
     def track(self, medium):
         """Start tracking calls made to a medium.
@@ -726,35 +724,49 @@ class SmartSimplePipesClientMedium(SmartClientStreamMedium):
 
     def _accept_bytes(self, bytes):
         """See SmartClientStreamMedium.accept_bytes."""
-        osutils.until_no_eintr(self._writeable_pipe.write, bytes)
+        self._writeable_pipe.write(bytes)
         self._report_activity(len(bytes), 'write')
 
     def _flush(self):
         """See SmartClientStreamMedium._flush()."""
-        osutils.until_no_eintr(self._writeable_pipe.flush)
+        self._writeable_pipe.flush()
 
     def _read_bytes(self, count):
         """See SmartClientStreamMedium._read_bytes."""
-        bytes = osutils.until_no_eintr(self._readable_pipe.read, count)
+        bytes_to_read = min(count, _MAX_READ_SIZE)
+        bytes = self._readable_pipe.read(bytes_to_read)
         self._report_activity(len(bytes), 'read')
         return bytes
 
 
-class SmartSSHClientMedium(SmartClientStreamMedium):
-    """A client medium using SSH."""
+class SSHParams(object):
+    """A set of parameters for starting a remote bzr via SSH."""
 
     def __init__(self, host, port=None, username=None, password=None,
-            base=None, vendor=None, bzr_remote_path=None):
+            bzr_remote_path='bzr'):
+        self.host = host
+        self.port = port
+        self.username = username
+        self.password = password
+        self.bzr_remote_path = bzr_remote_path
+
+
+class SmartSSHClientMedium(SmartClientStreamMedium):
+    """A client medium using SSH.
+    
+    It delegates IO to a SmartClientSocketMedium or
+    SmartClientAlreadyConnectedSocketMedium (depending on platform).
+    """
+
+    def __init__(self, base, ssh_params, vendor=None):
         """Creates a client that will connect on the first use.
 
+        :param ssh_params: A SSHParams instance.
         :param vendor: An optional override for the ssh vendor to use. See
             bzrlib.transport.ssh for details on ssh vendors.
         """
-        self._connected = False
-        self._host = host
-        self._password = password
-        self._port = port
-        self._username = username
+        self._real_medium = None
+        self._ssh_params = ssh_params
         # for the benefit of progress making a short description of this
         # transport
         self._scheme = 'bzr+ssh'
@@ -762,67 +774,70 @@ class SmartSSHClientMedium(SmartClientStreamMedium):
         # _DebugCounter so we have to store all the values used in our repr
         # method before calling the super init.
         SmartClientStreamMedium.__init__(self, base)
-        self._read_from = None
-        self._ssh_connection = None
         self._vendor = vendor
-        self._write_to = None
-        self._bzr_remote_path = bzr_remote_path
+        self._ssh_connection = None
 
     def __repr__(self):
-        if self._port is None:
+        if self._ssh_params.port is None:
             maybe_port = ''
         else:
-            maybe_port = ':%s' % self._port
+            maybe_port = ':%s' % self._ssh_params.port
         return "%s(%s://%s@%s%s/)" % (
             self.__class__.__name__,
             self._scheme,
-            self._username,
-            self._host,
+            self._ssh_params.username,
+            self._ssh_params.host,
             maybe_port)
 
     def _accept_bytes(self, bytes):
         """See SmartClientStreamMedium.accept_bytes."""
         self._ensure_connection()
-        osutils.until_no_eintr(self._write_to.write, bytes)
-        self._report_activity(len(bytes), 'write')
+        self._real_medium.accept_bytes(bytes)
 
     def disconnect(self):
         """See SmartClientMedium.disconnect()."""
-        if not self._connected:
-            return
-        osutils.until_no_eintr(self._read_from.close)
-        osutils.until_no_eintr(self._write_to.close)
-        self._ssh_connection.close()
-        self._connected = False
+        if self._real_medium is not None:
+            self._real_medium.disconnect()
+            self._real_medium = None
+        if self._ssh_connection is not None:
+            self._ssh_connection.close()
+            self._ssh_connection = None
 
     def _ensure_connection(self):
         """Connect this medium if not already connected."""
-        if self._connected:
+        if self._real_medium is not None:
             return
         if self._vendor is None:
             vendor = ssh._get_ssh_vendor()
         else:
             vendor = self._vendor
-        self._ssh_connection = vendor.connect_ssh(self._username,
-                self._password, self._host, self._port,
-                command=[self._bzr_remote_path, 'serve', '--inet',
+        self._ssh_connection = vendor.connect_ssh(self._ssh_params.username,
+                self._ssh_params.password, self._ssh_params.host,
+                self._ssh_params.port,
+                command=[self._ssh_params.bzr_remote_path, 'serve', '--inet',
                          '--directory=/', '--allow-writes'])
-        self._read_from, self._write_to = \
-            self._ssh_connection.get_filelike_channels()
-        self._connected = True
+        io_kind, io_object = self._ssh_connection.get_sock_or_pipes()
+        if io_kind == 'socket':
+            self._real_medium = SmartClientAlreadyConnectedSocketMedium(
+                self.base, io_object)
+        elif io_kind == 'pipes':
+            read_from, write_to = io_object
+            self._real_medium = SmartSimplePipesClientMedium(
+                read_from, write_to, self.base)
+        else:
+            raise AssertionError(
+                "Unexpected io_kind %r from %r"
+                % (io_kind, self._ssh_connection))
 
     def _flush(self):
         """See SmartClientStreamMedium._flush()."""
-        self._write_to.flush()
+        self._real_medium._flush()
 
     def _read_bytes(self, count):
         """See SmartClientStreamMedium.read_bytes."""
-        if not self._connected:
+        if self._real_medium is None:
             raise errors.MediumNotConnected(self)
-        bytes_to_read = min(count, _MAX_READ_SIZE)
-        bytes = osutils.until_no_eintr(self._read_from.read, bytes_to_read)
-        self._report_activity(len(bytes), 'read')
-        return bytes
+        return self._real_medium.read_bytes(count)
 
 
 # Port 4155 is the default port for bzr://, registered with IANA.
@@ -830,30 +845,59 @@ BZR_DEFAULT_INTERFACE = None
 BZR_DEFAULT_PORT = 4155
 
 
-class SmartTCPClientMedium(SmartClientStreamMedium):
-    """A client medium using TCP."""
+class SmartClientSocketMedium(SmartClientStreamMedium):
+    """A client medium using a socket.
+    
+    This class isn't usable directly.  Use one of its subclasses instead.
+    """
 
-    def __init__(self, host, port, base):
-        """Creates a client that will connect on the first use."""
+    def __init__(self, base):
         SmartClientStreamMedium.__init__(self, base)
-        self._connected = False
-        self._host = host
-        self._port = port
         self._socket = None
+        self._connected = False
 
     def _accept_bytes(self, bytes):
         """See SmartClientMedium.accept_bytes."""
         self._ensure_connection()
         osutils.send_all(self._socket, bytes, self._report_activity)
 
+    def _ensure_connection(self):
+        """Connect this medium if not already connected."""
+        raise NotImplementedError(self._ensure_connection)
+
+    def _flush(self):
+        """See SmartClientStreamMedium._flush().
+
+        For sockets we do no flushing. For TCP sockets we may want to turn off
+        TCP_NODELAY and add a means to do a flush, but that can be done in the
+        future.
+        """
+
+    def _read_bytes(self, count):
+        """See SmartClientMedium.read_bytes."""
+        if not self._connected:
+            raise errors.MediumNotConnected(self)
+        return osutils.read_bytes_from_socket(
+            self._socket, self._report_activity)
+
     def disconnect(self):
         """See SmartClientMedium.disconnect()."""
         if not self._connected:
             return
-        osutils.until_no_eintr(self._socket.close)
+        self._socket.close()
         self._socket = None
         self._connected = False
 
+
+class SmartTCPClientMedium(SmartClientSocketMedium):
+    """A client medium that creates a TCP connection."""
+
+    def __init__(self, host, port, base):
+        """Creates a client that will connect on the first use."""
+        SmartClientSocketMedium.__init__(self, base)
+        self._host = host
+        self._port = port
+
     def _ensure_connection(self):
         """Connect this medium if not already connected."""
         if self._connected:
@@ -893,19 +937,22 @@ class SmartTCPClientMedium(SmartClientStreamMedium):
                     (self._host, port, err_msg))
         self._connected = True
 
-    def _flush(self):
-        """See SmartClientStreamMedium._flush().
 
-        For TCP we do no flushing. We may want to turn off TCP_NODELAY and
-        add a means to do a flush, but that can be done in the future.
-        """
+class SmartClientAlreadyConnectedSocketMedium(SmartClientSocketMedium):
+    """A client medium for an already connected socket.
+    
+    Note that this class will assume it "owns" the socket, so it will close it
+    when its disconnect method is called.
+    """
 
-    def _read_bytes(self, count):
-        """See SmartClientMedium.read_bytes."""
-        if not self._connected:
-            raise errors.MediumNotConnected(self)
-        return _read_bytes_from_socket(
-            self._socket.recv, count, self._report_activity)
+    def __init__(self, base, sock):
+        SmartClientSocketMedium.__init__(self, base)
+        self._socket = sock
+        self._connected = True
+
+    def _ensure_connection(self):
+        # Already connected, by definition!  So nothing to do.
+        pass
 
 
 class SmartClientStreamMediumRequest(SmartClientMediumRequest):
@@ -948,19 +995,3 @@ class SmartClientStreamMediumRequest(SmartClientMediumRequest):
         self._medium._flush()
 
 
-def _read_bytes_from_socket(sock, desired_count, report_activity):
-    # We ignore the desired_count because on sockets it's more efficient to
-    # read large chunks (of _MAX_READ_SIZE bytes) at a time.
-    try:
-        bytes = osutils.until_no_eintr(sock, _MAX_READ_SIZE)
-    except socket.error, e:
-        if len(e.args) and e.args[0] in (errno.ECONNRESET, 10054):
-            # The connection was closed by the other side.  Callers expect an
-            # empty string to signal end-of-stream.
-            bytes = ''
-        else:
-            raise
-    else:
-        report_activity(len(bytes), 'read')
-    return bytes
-
