@@ -1,4 +1,4 @@
-# Copyright (C) 2007 Canonical Ltd
+# Copyright (C) 2007-2010 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -12,11 +12,14 @@
 #
 # You should have received a copy of the GNU General Public License
 # along with this program; if not, write to the Free Software
-# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 
 """Tests for commands related to tags"""
 
-from bzrlib import bzrdir
+from bzrlib import (
+    bzrdir,
+    tag,
+    )
 from bzrlib.branch import (
     Branch,
     )
@@ -47,6 +50,18 @@ class TestTagging(TestCaseWithTransport):
         self.assertContainsRe(err,
             "Tags can only be placed on a single revision")
 
+    def test_no_tag_name(self):
+        out, err = self.run_bzr('tag -d branch', retcode=3)
+        self.assertContainsRe(err, 'Please specify a tag name.')
+
+    def test_automatic_tag_name(self):
+        def get_tag_name(branch, revid):
+            return "mytag"
+        Branch.hooks.install_named_hook('automatic_tag_name',
+            get_tag_name, 'get tag name')
+        out, err = self.run_bzr('tag -d branch')
+        self.assertContainsRe(out, 'Created tag mytag.')
+
     def test_tag_current_rev(self):
         t = self.make_branch_and_tree('branch')
         t.commit(allow_pointless=True, message='initial commit',
@@ -73,6 +88,10 @@ class TestTagging(TestCaseWithTransport):
         # ... but can if you use --force
         out, err = self.run_bzr('tag -d branch NEWTAG --force')
 
+    def test_tag_delete_requires_name(self):
+        out, err = self.run_bzr('tag -d branch', retcode=3)
+        self.assertContainsRe(err, 'Please specify a tag name\\.')
+
     def test_branch_push_pull_merge_copies_tags(self):
         t = self.make_branch_and_tree('branch1')
         t.commit(allow_pointless=True, message='initial commit',
