@@ -1,4 +1,4 @@
-# Copyright (C) 2004, 2005 Canonical Ltd
+# Copyright (C) 2005-2010 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -16,6 +16,7 @@
 
 from cStringIO import StringIO
 import errno
+import inspect
 import sys
 
 from bzrlib import (
@@ -24,6 +25,7 @@ from bzrlib import (
     config,
     errors,
     option,
+    symbol_versioning,
     tests,
     )
 from bzrlib.commands import display_command
@@ -32,6 +34,17 @@ from bzrlib.tests import TestSkipped
 
 class TestCommands(tests.TestCase):
 
+    def test_all_commands_have_help(self):
+        commands._register_builtin_commands()
+        commands_without_help = set()
+        base_doc = inspect.getdoc(commands.Command)
+        for cmd_name in commands.all_command_names():
+            cmd = commands.get_cmd_object(cmd_name)
+            cmd_help = cmd.help()
+            if not cmd_help or cmd_help == base_doc:
+                commands_without_help.append(cmd_name)
+        self.assertLength(0, commands_without_help)
+
     def test_display_command(self):
         """EPIPE message is selectively suppressed"""
         def pipe_thrower():
@@ -64,7 +77,7 @@ class TestCommands(tests.TestCase):
     @staticmethod
     def get_command(options):
         class cmd_foo(commands.Command):
-            'Bar'
+            __doc__ = 'Bar'
 
             takes_options = options
 
@@ -111,7 +124,7 @@ class TestGetAlias(tests.TestCase):
 
     def test_unicode(self):
         my_config = self._get_config("[ALIASES]\n"
-            u"iam=whoami 'Erik B\u00e5gfors <erik@bagfors.nu>'\n")
+            u'iam=whoami "Erik B\u00e5gfors <erik@bagfors.nu>"\n')
         self.assertEqual([u'whoami', u'Erik B\u00e5gfors <erik@bagfors.nu>'],
                           commands.get_alias("iam", config=my_config))
 
@@ -119,38 +132,35 @@ class TestGetAlias(tests.TestCase):
 class TestSeeAlso(tests.TestCase):
     """Tests for the see also functional of Command."""
 
-    def test_default_subclass_no_see_also(self):
+    @staticmethod
+    def _get_command_with_see_also(see_also):
         class ACommand(commands.Command):
-            """A sample command."""
-        command = ACommand()
+            __doc__ = """A sample command."""
+            _see_also = see_also
+        return ACommand()
+
+    def test_default_subclass_no_see_also(self):
+        command = self._get_command_with_see_also([])
         self.assertEqual([], command.get_see_also())
 
     def test__see_also(self):
         """When _see_also is defined, it sets the result of get_see_also()."""
-        class ACommand(commands.Command):
-            _see_also = ['bar', 'foo']
-        command = ACommand()
+        command = self._get_command_with_see_also(['bar', 'foo'])
         self.assertEqual(['bar', 'foo'], command.get_see_also())
 
     def test_deduplication(self):
         """Duplicates in _see_also are stripped out."""
-        class ACommand(commands.Command):
-            _see_also = ['foo', 'foo']
-        command = ACommand()
+        command = self._get_command_with_see_also(['foo', 'foo'])
         self.assertEqual(['foo'], command.get_see_also())
 
     def test_sorted(self):
         """_see_also is sorted by get_see_also."""
-        class ACommand(commands.Command):
-            _see_also = ['foo', 'bar']
-        command = ACommand()
+        command = self._get_command_with_see_also(['foo', 'bar'])
         self.assertEqual(['bar', 'foo'], command.get_see_also())
 
     def test_additional_terms(self):
         """Additional terms can be supplied and are deduped and sorted."""
-        class ACommand(commands.Command):
-            _see_also = ['foo', 'bar']
-        command = ACommand()
+        command = self._get_command_with_see_also(['foo', 'bar'])
         self.assertEqual(['bar', 'foo', 'gam'],
             command.get_see_also(['gam', 'bar', 'gam']))
 
@@ -210,14 +220,13 @@ class TestExtendCommandHook(tests.TestCase):
         commands.Command.hooks.install_named_hook(
             "extend_command", hook_calls.append, None)
         # create a command, should not fire
-        class ACommand(commands.Command):
-            """A sample command."""
-        cmd = ACommand()
+        class cmd_test_extend_command_hook(commands.Command):
+            __doc__ = """A sample command."""
         self.assertEqual([], hook_calls)
         # -- as a builtin
         # register the command class, should not fire
         try:
-            builtins.cmd_test_extend_command_hook = ACommand
+            commands.builtin_command_registry.register(cmd_test_extend_command_hook)
             self.assertEqual([], hook_calls)
             # and ask for the object, should fire
             cmd = commands.get_cmd_object('test-extend-command-hook')
@@ -227,7 +236,7 @@ class TestExtendCommandHook(tests.TestCase):
             self.assertSubset([cmd], hook_calls)
             del hook_calls[:]
         finally:
-            del builtins.cmd_test_extend_command_hook
+            commands.builtin_command_registry.remove('test-extend-command-hook')
         # -- as a plugin lazy registration
         try:
             # register the command class, should not fire
@@ -249,7 +258,7 @@ class TestGetCommandHook(tests.TestCase):
         commands.install_bzr_command_hooks()
         hook_calls = []
         class ACommand(commands.Command):
-            """A sample command."""
+            __doc__ = """A sample command."""
         def get_cmd(cmd_or_None, cmd_name):
             hook_calls.append(('called', cmd_or_None, cmd_name))
             if cmd_name in ('foo', 'info'):
@@ -276,32 +285,45 @@ class TestGetCommandHook(tests.TestCase):
 
 class TestGetMissingCommandHook(tests.TestCase):
 
-    def test_fires_on_get_cmd_object(self):
-        # The get_missing_command(cmd) hook fires when commands are delivered to the
-        # ui.
-        hook_calls = []
+    def hook_missing(self):
+        """Hook get_missing_command for testing."""
+        self.hook_calls = []
         class ACommand(commands.Command):
-            """A sample command."""
+            __doc__ = """A sample command."""
         def get_missing_cmd(cmd_name):
-            hook_calls.append(('called', cmd_name))
+            self.hook_calls.append(('called', cmd_name))
             if cmd_name in ('foo', 'info'):
                 return ACommand()
         commands.Command.hooks.install_named_hook(
             "get_missing_command", get_missing_cmd, None)
+        self.ACommand = ACommand
+
+    def test_fires_on_get_cmd_object(self):
+        # The get_missing_command(cmd) hook fires when commands are delivered to the
+        # ui.
+        self.hook_missing()
         # create a command directly, should not fire
-        cmd = ACommand()
-        self.assertEqual([], hook_calls)
+        self.cmd = self.ACommand()
+        self.assertEqual([], self.hook_calls)
         # ask by name, should fire and give us our command
         cmd = commands.get_cmd_object('foo')
-        self.assertEqual([('called', 'foo')], hook_calls)
-        self.assertIsInstance(cmd, ACommand)
-        del hook_calls[:]
+        self.assertEqual([('called', 'foo')], self.hook_calls)
+        self.assertIsInstance(cmd, self.ACommand)
+        del self.hook_calls[:]
         # ask by a name that is supplied by a builtin - the hook should not
         # fire and we still get our object.
         commands.install_bzr_command_hooks()
         cmd = commands.get_cmd_object('info')
         self.assertNotEqual(None, cmd)
-        self.assertEqual(0, len(hook_calls))
+        self.assertEqual(0, len(self.hook_calls))
+
+    def test_skipped_on_HelpCommandIndex_get_topics(self):
+        # The get_missing_command(cmd_name) hook is not fired when
+        # looking up help topics.
+        self.hook_missing()
+        topic = commands.HelpCommandIndex()
+        topics = topic.get_topics('foo')
+        self.assertEqual([], self.hook_calls)
 
 
 class TestListCommandHook(tests.TestCase):
@@ -323,3 +345,10 @@ class TestListCommandHook(tests.TestCase):
         cmds = list(commands.all_command_names())
         self.assertEqual(['called'], hook_calls)
         self.assertSubset(['foo', 'bar'], cmds)
+
+class TestDeprecations(tests.TestCase):
+
+    def test_shlex_split_unicode_deprecation(self):
+        res = self.applyDeprecated(
+                symbol_versioning.deprecated_in((2, 2, 0)),
+                commands.shlex_split_unicode, 'whatever')
