@@ -89,6 +89,18 @@ def load_tests(standard_tests, module, loader):
                             _qualified_prefix='http+pycurl',)))
     tests.multiply_tests(t_tests, transport_scenarios, result)
 
+    protocol_scenarios = [
+            ('HTTP/1.0',  dict(_protocol_version='HTTP/1.0')),
+            ('HTTP/1.1',  dict(_protocol_version='HTTP/1.1')),
+            ]
+
+    # some tests are parametrized by the protocol version only
+    p_tests, remaining_tests = tests.split_suite_by_condition(
+        remaining_tests, tests.condition_isinstance((
+                TestAuthOnRedirected,
+                )))
+    tests.multiply_tests(p_tests, protocol_scenarios, result)
+
     # each implementation tested with each HTTP version
     tp_tests, remaining_tests = tests.split_suite_by_condition(
         remaining_tests, tests.condition_isinstance((
@@ -103,10 +115,6 @@ def load_tests(standard_tests, module, loader):
                 TestRanges,
                 TestSpecificRequestHandler,
                 )))
-    protocol_scenarios = [
-            ('HTTP/1.0',  dict(_protocol_version='HTTP/1.0')),
-            ('HTTP/1.1',  dict(_protocol_version='HTTP/1.1')),
-            ]
     tp_scenarios = tests.multiply_scenarios(transport_scenarios,
                                             protocol_scenarios)
     tests.multiply_tests(tp_tests, tp_scenarios, result)
@@ -391,10 +399,10 @@ class TestHttpUrls(tests.TestCase):
         self.assertEqual('http://example.com', url)
         self.assertEqual(0, len(f.credentials))
         url = http.extract_auth(
-            'http://user:pass@www.bazaar-vcs.org/bzr/bzr.dev', f)
-        self.assertEqual('http://www.bazaar-vcs.org/bzr/bzr.dev', url)
+            'http://user:pass@example.com/bzr/bzr.dev', f)
+        self.assertEqual('http://example.com/bzr/bzr.dev', url)
         self.assertEqual(1, len(f.credentials))
-        self.assertEqual([None, 'www.bazaar-vcs.org', 'user', 'pass'],
+        self.assertEqual([None, 'example.com', 'user', 'pass'],
                          f.credentials[0])
 
 
@@ -452,26 +460,22 @@ class TestHttps_pycurl(TestWithTransport_pycurl, tests.TestCase):
         # Import the module locally now that we now it's available.
         pycurl = features.pycurl.module
 
-        version_info_orig = pycurl.version_info
-        def restore():
-            pycurl.version_info = version_info_orig
-        self.addCleanup(restore)
-
-        # Fake the pycurl version_info This was taken from a windows pycurl
-        # without SSL (thanks to bialix)
-        pycurl.version_info = lambda : (2,
-                                        '7.13.2',
-                                        462082,
-                                        'i386-pc-win32',
-                                        2576,
-                                        None,
-                                        0,
-                                        None,
-                                        ('ftp', 'gopher', 'telnet',
-                                         'dict', 'ldap', 'http', 'file'),
-                                        None,
-                                        0,
-                                        None)
+        self.overrideAttr(pycurl, 'version_info',
+                          # Fake the pycurl version_info This was taken from
+                          # a windows pycurl without SSL (thanks to bialix)
+                          lambda : (2,
+                                    '7.13.2',
+                                    462082,
+                                    'i386-pc-win32',
+                                    2576,
+                                    None,
+                                    0,
+                                    None,
+                                    ('ftp', 'gopher', 'telnet',
+                                     'dict', 'ldap', 'http', 'file'),
+                                    None,
+                                    0,
+                                    None))
         self.assertRaises(errors.DependencyNotPresent, self._transport,
                           'https://launchpad.net')
 
@@ -1364,11 +1368,7 @@ class RedirectedRequest(_urllib2_wrappers.Request):
 
 
 def install_redirected_request(test):
-    test.original_class = _urllib2_wrappers.Request
-    def restore():
-        _urllib2_wrappers.Request = test.original_class
-    _urllib2_wrappers.Request = RedirectedRequest
-    test.addCleanup(restore)
+    test.overrideAttr(_urllib2_wrappers, 'Request', RedirectedRequest)
 
 
 class TestHTTPSilentRedirections(http_utils.TestCaseWithRedirectedWebserver):
@@ -2164,13 +2164,14 @@ class TestAuthOnRedirected(http_utils.TestCaseWithRedirectedWebserver):
     _transport = _urllib.HttpTransport_urllib
 
     def create_transport_readonly_server(self):
-        return self._auth_server()
+        return self._auth_server(protocol_version=self._protocol_version)
 
     def create_transport_secondary_server(self):
         """Create the secondary server redirecting to the primary server"""
         new = self.get_readonly_server()
 
-        redirecting = http_utils.HTTPServerRedirecting()
+        redirecting = http_utils.HTTPServerRedirecting(
+            protocol_version=self._protocol_version)
         redirecting.redirect_to(new.host, new.port)
         return redirecting
 
