@@ -20,10 +20,9 @@
 
 import types
 
-
+from bzrlib import transport
 from bzrlib.tests import TestCaseWithMemoryTransport
 from bzrlib.trace import mutter
-from bzrlib.transport import get_transport
 from bzrlib.transport.log import TransportLogDecorator
 
 
@@ -31,7 +30,8 @@ class TestTransportLog(TestCaseWithMemoryTransport):
 
     def test_log_transport(self):
         base_transport = self.get_transport('')
-        logging_transport = get_transport('log+' + base_transport.base)
+        logging_transport = transport.get_transport(
+            'log+' + base_transport.base)
 
         # operations such as mkdir are logged
         mutter('where are you?')
