@@ -23,7 +23,7 @@
 # ElementTree bits
 
 from bzrlib.serializer import Serializer
-from bzrlib.trace import mutter, warning
+from bzrlib.trace import mutter
 
 try:
     try:
@@ -31,8 +31,6 @@ try:
         from xml.etree.cElementTree import (ElementTree, SubElement, Element,
             XMLTreeBuilder, fromstring, tostring)
         import xml.etree as elementtree
-        # Also import ElementTree module so monkey-patching below always works
-        import xml.etree.ElementTree
     except ImportError:
         from cElementTree import (ElementTree, SubElement, Element,
                                   XMLTreeBuilder, fromstring, tostring)
@@ -85,8 +83,11 @@ class XMLSerializer(Serializer):
 
     def read_inventory(self, f, revision_id=None):
         try:
-            return self._unpack_inventory(self._read_element(f),
-                revision_id=None)
+            try:
+                return self._unpack_inventory(self._read_element(f),
+                    revision_id=None)
+            finally:
+                f.close()
         except ParseError, e:
             raise errors.UnexpectedInventoryFormat(e)
 
